/*
  Copyright (C) 2005 Steven L. Scott

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
#ifndef BOOM_SLICE_SAMPLER_HPP
#define BOOM_SLICE_SAMPLER_HPP

#include <Samplers/Sampler.hpp>
#include <LinAlg/Vector.hpp>
#include <boost/function.hpp>

namespace BOOM{

  typedef boost::function<double(const Vector &)> Func;

  class SliceSampler : public Sampler{
  public:
    ~SliceSampler() override{}
    SliceSampler(Func F, bool unimodal=false);
    Vector draw(const Vector &x) override;
    double logp(const Vector &x)const;
  private:
    double lo, hi, plo, phi, pstar, scale;
    Vector theta, z;
    bool unimodal;
    Func f;
    void doubling(bool);
    void contract(double lam, double p);
    void find_limits();
    void random_direction();
    void initialize();
  };


}

#endif// BOOM_SLICE_SAMPLER_HPP
